import tempfile
from enum import Enum
from tcheckerpy.utils import call_tchecker

class Algorithm(Enum):
    COUVSCC = 0 # Couvreur's SCC-decomposition-based algorithm (search an accepting cycle that visits all labels)
    NDFS = 1 # nested depth-first search algorithm over the zone graph (search an accepting cycle with a state with all labels)

class Certificate(Enum):
    GRAPH = 0 # graph of explored state-space
    SYMBOLIC = 1 # symbolic lasso run with loop on labels (not for couvscc with multiple labels)
    NONE = 2 # no certificate

def liveness(sys_decl: str, algorithm: Algorithm, certificate: Certificate = Certificate.NONE, 
             labels: list[str] = [], block_size: int | None = None, table_size: int | None = None) -> tuple[bool, str, str]:
    """
    Checks whether timed automaton contains a cycle.

    :param sys_decl: system declaration of timed automaton
    :param algorithm: algorithm to be used (see `tck_liveness.Algorithm`)
    :param certificate: certificate type (see `tck_liveness.Certificate`)
    :param labels: list of accepting labels
    :param block_size: block size for internal computation
    :param table_size: table size for internal computation
    :return: result of liveness check (True iff timed automaton contains cycle), statistics and certificate
    """

    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(sys_decl.encode('utf-8'))
        temp_file_path = temp_file.name
    
    # convert list to string
    labels_str = ", ".join(labels)

    # call the TChecker function
    stats, cert = call_tchecker.call_tchecker_function_in_new_process(
        func_name="tck_liveness",
        argtypes=["ctypes.c_char_p", "ctypes.c_char_p", "ctypes.c_int", "ctypes.c_int",
                  "ctypes.POINTER(ctypes.c_int)", "ctypes.POINTER(ctypes.c_int)"],
        has_result=True,
        args=[temp_file_path, labels_str, algorithm.value, certificate.value,
              block_size, table_size]
    )

    return "CYCLE true" in stats, stats, cert



#!/usr/bin/env python3
import ctypes
import json
import argparse

def main():
    parser = argparse.ArgumentParser(
        description="Load TChecker shared library and call a function with specified arguments."
    )
    parser.add_argument(
        "--lib-path", "-L",
        required=True,
        help="Path to the shared library (e.g. ./libtchecker.so)"
    )
    parser.add_argument(
        "--func-name", "-f",
        required=True,
        help="Name of the exported function to call"
    )
    parser.add_argument(
        "--argtypes", "-a",
        required=True,
        help="JSON-encoded list of ctypes type names, e.g. '[\"ctypes.c_int\", \"ctypes.c_double\"]'"
    )
    parser.add_argument(
        "--args", "-A",
        required=True,
        help="JSON-encoded list of argument values"
    )
    opts = parser.parse_args()

    # 1) load the library
    dll = ctypes.CDLL(opts.lib_path)

    # 2) grab the function
    func = getattr(dll, opts.func_name)

    # 3) set its signature
    argtypes = json.loads(opts.argtypes)
    func.argtypes = [eval(t) for t in argtypes]

    # 4) parse our JSON args
    py_args = json.loads(opts.args)

    for idx, at in enumerate(func.argtypes):
        if at is ctypes.c_char_p:
            val = py_args[idx]
            if isinstance(val, str):
                py_args[idx] = val.encode('utf-8')
        if at is ctypes.POINTER(ctypes.c_int):
            val = py_args[idx]
            if isinstance(val, int):
                py_args[idx] = ctypes.pointer(ctypes.c_int(val))
            elif isinstance(val, list):
                py_args[idx] = (ctypes.c_int * len(val))(*val)

    # 5) call it (any C‑side prints still go to stdout)
    func(*py_args)

if __name__ == "__main__":
    main()



import tempfile
from tcheckerpy.utils import call_tchecker

def __call_tck_syntax(sys_decl: str, func_name: str, process_name: str | None = None) -> str:

    with tempfile.NamedTemporaryFile(delete=False) as temp_file:
        temp_file.write(sys_decl.encode('utf-8'))
        temp_file_path = temp_file.name

    product = func_name == "tck_syntax_create_synchronized_product"
        
    # call TChecker function
    _, result = call_tchecker.call_tchecker_function_in_new_process(
        func_name = func_name,
        argtypes = ["ctypes.c_char_p", "ctypes.c_char_p"] if product else ["ctypes.c_char_p"],
        has_result = True,
        args = [temp_file_path, process_name] if product else [temp_file_path]
    )

    return result

def check(sys_decl: str) -> None:
    """
    Check syntax of timed automaton.

    :param sys_decl: system declaration of timed automaton
    :raises RuntimeError: if syntax is incorrect
    """ 
    __call_tck_syntax(sys_decl, "tck_syntax_check_syntax")

def to_dot(sys_decl: str) -> str:
    """
    Convert timed automaton to DOT graphviz format.

    :param sys_decl: system declaration of timed automaton
    :return: system declaration in DOT graphviz format
    """
    return __call_tck_syntax(sys_decl, "tck_syntax_to_dot")

def to_json(sys_decl: str) -> str:
    """
    Convert timed automaton to JSON format.

    :param sys_decl: system declaration of timed automaton
    :return: system declaration in JSON format
    """
    return __call_tck_syntax(sys_decl, "tck_syntax_to_json")

def create_product(sys_decl: str, process_name: str = "P") -> str:
    """
    Create a synchronized product of timed automaton with multiple processes.

    :param sys_decl: system declaration of timed automaton
    :param process_name: name of synchronized process
    :return: system declaration of synchronized product
    """
    return __call_tck_syntax(sys_decl, "tck_syntax_create_synchronized_product", process_name)



import subprocess
import os
import sys
import json
from typing import List, Tuple, Any
import tempfile
from typing import Optional

def call_tchecker_function_in_new_process(
    func_name: str,
    argtypes: List[str],
    has_result: bool,
    args: List[Any],
    lib_path: str = os.path.join(os.path.dirname(__file__), "../libtchecker.so"),
    caller_script: str = os.path.join(os.path.dirname(__file__), "tchecker_caller.py")
) -> Tuple[str, Optional[str]]:
    """
    Calls a function in a fresh Python subprocess by invoking tchecker_caller.py.

    Returns stdout_output (everything before the final line)
    and result (parsed from the final line).
    """

    # If the function has a result the result gets written to a file with the file name passed as the first argument
    if has_result:
        # Create a temporary file to store the result
        with tempfile.NamedTemporaryFile(delete=False) as temp_file:
            result_filename = temp_file.name
            args = [result_filename] + args
            argtypes = ["ctypes.c_char_p"] + argtypes

    args_json     = json.dumps(args)
    argtypes_json = json.dumps(argtypes)

    cmd = [
        sys.executable,
        caller_script,
        "--lib-path", lib_path,
        "--func-name", func_name,
        "--argtypes", argtypes_json,
        "--args", args_json,
    ]

    proc = subprocess.run(cmd, capture_output=True, text=True)

    if proc.returncode != 0 or "ERROR" in proc.stderr:
        raise RuntimeError(f"Child process failed: {proc.stderr.strip()}, returncode: {proc.returncode}")

    result = None
    if has_result:
        with open(result_filename, "r") as result_file:
            result = result_file.read()

    return proc.stdout, result